function Publish(DataPaths, ID, figure, options)
%Publishes saves plot, data and measuring script 
%   Location sets the storage location. 'local' sets the storage location
%   to the current folder (an export folder will be created), 'server' is a
%   remote path, that is defined in the config file.
%   Two Methods are implemented 'individual' stores the data for
%   each plot while 'centralized' uses a data folder and uses reference links
%   to the original data (hdf5 only).
%   ParentFolder is the folder Name where the exported data is stored if an
%   path is used, PlotId will use this path a storagePath

arguments
   DataPaths
   ID (1,:) {mustBeNonzeroLengthText} % ID must be provided
   figure (1,:) {mustBeFigure} % Checks if figure is a figure object
   options.Location {mustBeMember(options.Location ,{'local','server','manual','CI-Test'})} = 'local' % storage path
   options.Method {mustBeMember(options.Method ,{'individual','centralized'})} = 'individual'
   options.ParentFolder (1,:) {mustBeText} = 'export'
   options.CopyUserFCN (1,1) {mustBeNumericOrLogical} = true 
   options.CSV (1,1) {mustBeNumericOrLogical} = false
end

%catch multiple figures in fig
if numel(figure) > 1
    figure = figure(1);
    msg = ['Publish is designed for handeling one figure at once' newline,...
        '- publishes uses only the first figure.' newline , ...
        'Consider an external for loop for multi figure export as provided in example.m'];
    warning(msg);
end

%% read config file
try 
   txt = fileread('config.json');
   config = jsondecode(txt);
   configError = false;
catch
   msg = ['Error while reading the config file' newline,...
       ' publishing on server not possible'];
   warning(msg);
   configError = true;
end

%% storage location
switch options.Location 
    case 'local'
        if contains(options.ParentFolder, {'/','\'})
            storPath = options.ParentFolder;
        else
            % use the script path as export path
            scriptPath = fileparts(DataPaths.script);
            storPath = fullfile(scriptPath,options.ParentFolder);
        end
    case 'server' %from config File
         storPath = config.ServerPath;
    case 'manual' %UI
        storPath = uigetdir();
    case 'CI-Test'
        storPath = fullfile(pwd,'CI_files',options.ParentFolder);
end
folderName = char(ID);

%% Create Data-Directory
if isfolder(fullfile(storPath,folderName))
   error(['Folder ',folderName, ' exists - Plot was already published ']);
elseif mkdir(fullfile(storPath,folderName))
else
    error('Directory could not be created - check remote path and permissions');
end
disp(['publishing of ', ID, ' started']);

%% Create a Copy of the script and user functions(optional)
% script
PlotID.createFileCopy({[DataPaths.script,'.m']},folderName,storPath,ID, 'script');

% user functions
[fList,pList] = matlab.codetools.requiredFilesAndProducts(DataPaths.script);
if options.CopyUserFCN 
   fList = fList(~ismember(fList,[DataPaths.script,'.m'])); % rmv script from list
   fList = fList(~contains(fList,'config.json')); % rmv config.json from list
   fList = PlotID.removePltIdFiles(fList); % Do not copy files that are part of plot ID
   if ~isempty(fList)
       PlotID.createFileCopy(fList,folderName,storPath,ID,'userFcn');
   end
end

%% Research data handeling
switch options.Method
    case 'centralized' 
        DataFolderName = 'data';
        % check if data folder exists
        if ~isfolder(fullfile(storPath,DataFolderName))
            mkdir(fullfile(storPath,DataFolderName));
        end
        % to get relative Paths
        currentPath = fullfile(storPath);
        
        %list all files
        fList = dir(fullfile(storPath,DataFolderName, '**\*.*'));
        %get list of files and folders in any subfolder
        fList = fList(~[fList.isdir]);  %remove folders from list
        fList = struct2table(fList);
                
        % Check if the new plot is based on the original data-set    
            % copy the data(once)
        for i=1:numel(DataPaths.rdata)            
            % check if identical file exists (status = 1)
            [~, idx] = PlotID.fileCompare(DataPaths.rdata{i},fList);
            % create Linked HDF5 files for identical files
            if any(idx)
                fList.path = fullfile(fList.folder,fList.name);
                sourcePath = fList{idx,'path'};
                relativeSourcePath = strrep(sourcePath,currentPath,'');
                                
                if contains(sourcePath,{'.h5','.hdf5'}) % Linking only for HDF5
                    PlotID.createLinkedHDF5(relativeSourcePath{1,1},storPath,ID);
                end
            else % no identical file exists
                %Copy the file in data and create the links (if hdf5)
                [dataPath] = PlotID.createFileCopy(DataPaths.rdata{i},'data',storPath,ID,'dataCentral');
                relativeDataPath = strrep(dataPath,currentPath,'');
                %WIP
                if contains(DataPaths.rdata{i},{'.h5','.hdf5'}) % Linking only for HDF5
                   % and create also linked files in the plot folder 
                   PlotID.createLinkedHDF5(relativeDataPath,storPath,ID);
                end  %if
            end %if
        end %for
        clear DataFolderName 
    case 'individual'
        % Create a copy of the research data    
        PlotID.createFileCopy(DataPaths.rdata,folderName,storPath,ID, 'data');
end
%% Write Config File

if ~configError %config File must exist
   % copy config file 
   configPath = PlotID.createFileCopy('config.json',folderName,...
       storPath,ID, 'data');
else
   configPath = fullfile(storPath,folderName,'config.json');
   config = struct(); 
   if ispc
    config.author = getenv('USERNAME');
   end
end
% add further Metadata
config.ProjectID = ID;
config.CreationDate = datestr(now);
config.MatlabVersion = version;
config.ToolboxVersions = pList;

%write config 
fid = fopen(char(configPath),'w');
txt = jsonencode(config,'PrettyPrint',true);
fprintf(fid,txt); 
fclose(fid);


%% Export the Plot
try 
       PlotName = [ID,'_plot']; % plotname
       RemotePath = fullfile(storPath ,folderName, PlotName);
       % Matlab figure
       savefig(figure,RemotePath);
       % the png should only be a preview
       exportgraphics(figure,[RemotePath,'.png'],'Resolution',300);
catch 
    warning('Plot export was not successful')
end

disp(['publishing of ', ID , ' done']);

% CSV EXport
if options.CSV
  T = table();
  T.research_Data = DataPaths.rdata'; T.PlotID(:) = {ID};
  T.Script_Name(:) = {[DataPaths.script,'.m']};
  T.Storage_Location(:) = {storPath};
  T.Date(:) = {datestr(now)};
  T = movevars(T,'PlotID','before',1);
  writetable(T, fullfile(storPath, 'overview_table.csv'),'WriteMode','append');
end

end %function

function tf = mustBeFigure(h)
%checks if input is a figure object
  tf = strcmp(get(h, 'type'), 'figure') & isa(h, 'matlab.ui.Figure');  
end
