function [figs, IDs] = TagPlot(figs, options)
%TagPlot adds IDs to figures
%   The ID is placed visual on the figure window and as Tag (property of figure)
%   TagPlot can tag multiple figures at once.
%   If a single Plot is taged IDs is a char, otherwise it is a cell array of
%   chars 
%   options.ProjectID is the project number (string or char), if empty the ID from the
%   config file is used
%   
%   The ID is placed on the 'east' per default, if you want it somwhere
%   else, use the 'Location' option. 'north','east','south','west' are
%   predefined, otherwise use the 'custom' option and provide the desired
%   'Position' (relative to your x- and y-axis limits)
arguments
    figs (1,:) {mustBeFigure}
    options.ProjectID (1,:) {mustBeText}= ''
    options.Fontsize (1,1) {mustBeInteger} = 8 
    options.Location (1,:) {mustBeText} = 'east'
    options.Position (1,2) {mustBeVector} = [1,0.4] % default for east
    options.Rotation (1,1) {mustBeInteger} = 0  
end

if isempty(options.ProjectID)
    try
        txt = fileread('config.json');
        config = jsondecode(txt);
    catch
        config =struct;
        config.ProjectID = '';
        warning("No ProjectID was definded and no config.json could be found");
        
    end
   
   if ~isempty(config.ProjectID)
       options.ProjectID = config.ProjectID;
   else
       warning('no project options.ProjectID defined')
   end
end

switch options.Location
    case 'north'
        options.Position = [0.4,0.95];
        options.Rotation = 0;
    case 'east'
        options.Position = [1,0.4];
        options.Rotation = 90;
    case 'south'
        options.Position = [0.4,.02];
        options.Rotation = 0;
    case 'west'
        options.Position = [.05,0.4];
        options.Rotation = 90;
    case 'custom'
        % Check if Position is valid
        if ~all(0 <= options.Position & options.Position <= 1)
           options.Position = [1,0.4];
           warning('options.Position is not valid, TagPlot uses default values instead');
        end
    otherwise % set default position
        warning([options.Location, ' is not a defined location, TagPlot uses location east instead']);
        options.Location = 'east'; options.Position = [1,0.4];
end

IDs = cell(numel(figs),1);

for n = 1:numel(figs)
    IDs{n} = PlotID.CreateID; % Create ID
    IDs{n} = [options.ProjectID,'-',IDs{n}]; % add options.ProjectID
    axes = get(figs(n),'CurrentAxes'); % Axes object for text annotation
    % Limits for relative Positioning
    ylim =get(axes,'YLim');
    xlim =get(axes,'XLim');
    %ID
    position = [options.Position(1)*xlim(2), options.Position(2)*ylim(2)];
    text(axes,position(1),position(2), IDs{n},'Fontsize',options.Fontsize,...
    'Rotation',options.Rotation, 'VerticalAlignment','bottom','Color',...
        0.65*[1 1 1],'BackgroundColor','w');
    set(figs(n),'Tag', IDs{n});    
end

if numel(figs) == 1
   % use char instead of a cell arry for a single ID
   IDs = IDs{1}; 
end

end

function tf = mustBeFigure(h) %checks if input is a figure object
  tf = strcmp(get(h, 'type'), 'figure') & isa(h, 'matlab.ui.Figure');
end
